#!/bin/bash

# redirect all output to log file and console
{
    CURL_PARAMS="-s -w "%{response_code}""
    BASE_URL="/api/reader-agents/v1"
    
    echo "Current directory is ${PWD}"
    
    # get base directory
    if [ ! -z "$1" ]; then
      cd "$1" || exit
      echo "Directory changed to ${PWD}"
    fi
    
    if [ -f Staging ]; then
      export ENVIRONMENT="Staging"
      export ENVIRONMENT_LOWERCASE="staging"
      export BACKEND="https://staging.nedap-harmony.com"
      SERVER="https://staging.nedap-harmony.com"
    else
      export ENVIRONMENT="Production"
      export ENVIRONMENT_LOWERCASE="production"
      SERVER="https://nedap-harmony.com"
    fi
    
    # those variables should be set in reader-specific script
    echo "Script run for reader ${DEVICE}"
    
    # variable needed to execute dotnet on limited Linux installation
    export DOTNET_SYSTEM_GLOBALIZATION_INVARIANT=true
    
    BASE_VERSION=$(<version)
    echo "Base version is $BASE_VERSION"
    
    echo "Environment is set to $ENVIRONMENT"
    echo "Backend server is set to $SERVER"
    
    while :
    do
        unset SHOULD_VERSION
        unset SHOULD_MD5
        unset DIRECTORY
        unset CALCULATED_MD5
        unset HARMONY_READER_PATH
        
        # if Impinj reader, set the upgrade LED
        if [ "${DEVICE}" == "Impinj" ]; then
            echo "Impinj device, enable upgrade LED"
            echo 255 > /sys/devices/platform/leds/leds/upgrade-status/brightness
        fi
        
        # always default to base version
        BOOT_VERSION="BASE"
        
        # check if there is hardware information available from a connection to the reader
        if [ -s .hardwareInformation ]; then
            echo "HardwareInformation is present"
            . .hardwareInformation
            echo "Reader is ${BRAND} ${MODEL}, with serial ${SERIAL} and macAddress ${MAC_ADDRESS}"
            
            # if so, use this to get the version that is relevant for this reader
            STATUS_CODE_SHOULD_VERSION=$(curl $CURL_PARAMS -o .shouldVersion -G --data-urlencode "brand=${BRAND}" --data-urlencode "model=${MODEL}" --data-urlencode "serial=${SERIAL}" --data-urlencode "macAddress=${MAC_ADDRESS}" "${SERVER}${BASE_URL}/version")
            
            if [ "$STATUS_CODE_SHOULD_VERSION" != "200" ]; then
                echo "Should version NOT downloaded from server (HTTP status ${STATUS_CODE_SHOULD_VERSION}), trying offline file"
            fi
            
            if [ -s .shouldVersion ]; then
                SHOULD_VERSION=$(<.shouldVersion)
                echo "ShouldVersion is $SHOULD_VERSION"
                
                # if should version does not equal the base version, check for updated version
                if [ "$SHOULD_VERSION" != "base" ] && [ "$BASE_VERSION" != "$SHOULD_VERSION" ]; then
                    echo "Should version is not equal to the base version"
                    
                    # get the MD5 for this version
                    STATUS_CODE_MD5=$(curl $CURL_PARAMS -o .shouldMd5 "${SERVER}${BASE_URL}/${SHOULD_VERSION}/md5")
                    
                    if [ "$STATUS_CODE_MD5" != "200" ]; then
                        echo "Should MD5 NOT downloaded from server (HTTP status ${STATUS_CODE_MD5}), trying offline file"
                    fi
                    
                    if [ -s .shouldMd5 ]; then
                        SHOULD_MD5=$(<.shouldMd5)
                        echo "MD5 should be $SHOULD_MD5"
                        
                        # now check if there is a binary folder
                        DIRECTORY="update/HarmonyReader_${SHOULD_VERSION}"
                        echo "Update directory is $DIRECTORY"
                        
                        if [ ! -d $DIRECTORY ]; then
                            echo "Directory NOT found of shouldVersion, downloading binary"
                            
                            # now download the binary
                            STATUS_CODE_BINARY=$(curl $CURL_PARAMS -o /tmp/binary-${ENVIRONMENT_LOWERCASE}.zip "${SERVER}${BASE_URL}/${SHOULD_VERSION}/binary")
                            
                            if [ "$STATUS_CODE_BINARY" == "200" ]; then
                                echo "Binary downloaded successfully"
                                
                               if [ "$STATUS_CODE_MD5" == "200" ]; then
                                  echo "Binary and MD5 downloaded, installing update"
                                # remove all other update versions
                                rm -rf update
                                
                                # unzip new update version
                                unzip -o -q /tmp/binary-${ENVIRONMENT_LOWERCASE}.zip
                                
                                # remove the binary file, to make sure that we don't overflow memory space
                                rm /tmp/binary-${ENVIRONMENT_LOWERCASE}.zip
                                
                                else
                                  echo "successfully obtained binary, but failed to get the corresponding MD5"
                                fi
                            else
                                echo "Error in obtaining binary, http status code ${STATUS_CODE_BINARY}"
                            fi
                        fi
                        
                        if [ -d $DIRECTORY ]; then
                            echo "Directory found of shouldVersion"
                            
                            echo $(find $DIRECTORY -type f \( ! -iname ".*" \))
                            echo $(find $DIRECTORY -type f \( ! -iname ".*" \) -exec md5sum {} \;)
                            echo $(find $DIRECTORY -type f \( ! -iname ".*" \) -exec md5sum {} \; | sort -k 2)
                            echo $(find $DIRECTORY -type f \( ! -iname ".*" \) -exec md5sum {} \; | sort -k 2 | md5sum)
                            
                            # calculate MD5 over installed directory, exclude all dotfiles to ignore files written by app
                            # Step 1: find all files in the directory
                            # Step 2: calculate MD5sum over those files
                            # Step 3: sort those files on the second column, case-sensitive (LC_COLLATE=C)
                            # Step 4: calculate MD5sum over all those files
                            # Step 5: strip additional output from MD5sum output
                            CALCULATED_MD5=$(find $DIRECTORY -type f \( ! -iname ".*" \) -exec md5sum {} \; | LC_COLLATE=C sort -k 2 | md5sum | cut -c -32)
                            
                            echo "Calculated MD5 is ${CALCULATED_MD5}"
                            
                            # if the MD5 matches, than set the correct version to run
                            if [ "${CALCULATED_MD5}" == "${SHOULD_MD5}" ]; then
                                echo "Boot update version"
                                BOOT_VERSION="UPDATE"
                            else
                                echo "Invalid MD5, removing should version directory"
                                
                                # then remove shouldVersion to try again later
                                if [ -f .shouldVersion ]; then
                                    rm .shouldVersion
                                fi

                                if [ -f .shouldMd5 ]; then
                                    rm .shouldMd5
                                fi
                                
                                if [ -d update ]; then
                                    rm -rf update
                                fi
                                
                                # sleep to prevent infinite loops
                                sleep 30
                                
                                # if the MD5 does not match, but was successfully retrieved, try to download the binary again
                                echo "The retrieved MD5 did not match the calculated MD5, retrying update..."
                                continue
                            fi
                        fi
                    else
                        echo "No shouldMd5 defined"
                    fi
                else
                    echo "Should version equals base version, just execute base version"
                fi
            else
                echo "No shouldVersion defined"
            fi
        else
          echo "No hardwareInformation defined"
        fi
        
        # if Impinj reader, set the upgrade LED
        if [ "${DEVICE}" == "Impinj" ]; then
            echo "Impinj device, disable upgrade LED"
            echo 0 > /sys/devices/platform/leds/leds/upgrade-status/brightness
        fi
        
        echo "Booting version ${BOOT_VERSION}"
        HARMONY_READER_PATH="./"
        
        if [ ${BOOT_VERSION} == "UPDATE" ]; then
            HARMONY_READER_PATH="update/HarmonyReader_${SHOULD_VERSION}/"
        fi
        
        "${HARMONY_READER_PATH}HarmonyReader"
        
        EXIT_CODE=$?
        echo "Exit code was ${EXIT_CODE}"
        
        # check exit code for update request
        if [ ${EXIT_CODE} -eq 33 ]; then
            echo "Update requested, because of the specific exit code 33"
        elif [ ${EXIT_CODE} -eq 0 ]; then
            echo "Regular exit"
        else
            # somehow the application exited, which is not expected
            echo "Unexpected exit"
            
            # remove all traces of the update application such that the next boot will be defined by the server, or the base install
            if [ -f .shouldVersion ]; then
                rm .shouldVersion
            fi
            
            if [ -f .shouldMd5 ]; then
                rm .shouldMd5
            fi
            
            if [ -d update ]; then
                rm -rf update
            fi
            
            # sleep to prevent infinite loops
            sleep 30
        fi
    done
} 2>&1 | tee /tmp/HarmonyReader-$RANDOM-runscriptLog
